@@ -2838,7 +2838,9 @@ void ppc_opc_vxor()
 
 #define CR_CR6		(0x00f0)
 #define CR_CR6_EQ	(1<<7)
+#define CR_CR6_NE_SOME	(1<<6)
 #define CR_CR6_NE	(1<<5)
+#define CR_CR6_EQ_SOME	(1<<4)
 
 /*	vcmpequbx	Vector Compare Equal-to Unsigned Byte
  *	v.160
@@ -2854,9 +2856,11 @@ void ppc_opc_vcmpequbx()
 		if (gCPU.vr[vrA].b[i] == gCPU.vr[vrB].b[i]) {
 			gCPU.vr[vrD].b[i] = 0xff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].b[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -2880,9 +2884,11 @@ void ppc_opc_vcmpequhx()
 		if (gCPU.vr[vrA].h[i] == gCPU.vr[vrB].h[i]) {
 			gCPU.vr[vrD].h[i] = 0xffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].h[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -2906,9 +2912,11 @@ void ppc_opc_vcmpequwx()
 		if (gCPU.vr[vrA].w[i] == gCPU.vr[vrB].w[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -2932,9 +2940,11 @@ void ppc_opc_vcmpeqfpx()
 		if (gCPU.vr[vrA].f[i] == gCPU.vr[vrB].f[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -2958,9 +2968,11 @@ void ppc_opc_vcmpgtubx()
 		if (gCPU.vr[vrA].b[i] > gCPU.vr[vrB].b[i]) {
 			gCPU.vr[vrD].b[i] = 0xff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].b[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -2984,9 +2996,11 @@ void ppc_opc_vcmpgtsbx()
 		if (gCPU.vr[vrA].sb[i] > gCPU.vr[vrB].sb[i]) {
 			gCPU.vr[vrD].b[i] = 0xff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].b[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3010,9 +3024,11 @@ void ppc_opc_vcmpgtuhx()
 		if (gCPU.vr[vrA].h[i] > gCPU.vr[vrB].h[i]) {
 			gCPU.vr[vrD].h[i] = 0xffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].h[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3036,9 +3052,11 @@ void ppc_opc_vcmpgtshx()
 		if (gCPU.vr[vrA].sh[i] > gCPU.vr[vrB].sh[i]) {
 			gCPU.vr[vrD].h[i] = 0xffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].h[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3062,9 +3080,11 @@ void ppc_opc_vcmpgtuwx()
 		if (gCPU.vr[vrA].w[i] > gCPU.vr[vrB].w[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3088,9 +3108,11 @@ void ppc_opc_vcmpgtswx()
 		if (gCPU.vr[vrA].sw[i] > gCPU.vr[vrB].sw[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3114,9 +3136,11 @@ void ppc_opc_vcmpgtfpx()
 		if (gCPU.vr[vrA].f[i] > gCPU.vr[vrB].f[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
@@ -3140,9 +3164,11 @@ void ppc_opc_vcmpgefpx()
 		if (gCPU.vr[vrA].f[i] >= gCPU.vr[vrB].f[i]) {
 			gCPU.vr[vrD].w[i] = 0xffffffff;
 			tf &= ~CR_CR6_NE;
+			tf |= CR_CR6_EQ_SOME;
 		} else {
 			gCPU.vr[vrD].w[i] = 0;
 			tf &= ~CR_CR6_EQ;
+			tf |= CR_CR6_NE_SOME;
 		}
 	}
 
